<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use SimpleXMLIterator;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;

class ObjectParser implements ItemParser
{

    public const TAG = 'object';

    /**
     * Contains a group of parsers
     * @var ItemParser[]
     */
    private $parsers;
    
    /**
     * ObjectParser constructor.
     *
     * @since 0.1
     *
     * @param ItemParser[]
     */
    public function __construct(array $parsers)
    {
        $this->parsers = $parsers;
    }

    private function getType(SimpleXMLIterator $el)
    {
        return Arr::get(Arr::get((array)$el->attributes(), '@attributes', []), 'type', '');
    }

    private function getParser(SimpleXMLIterator $el)
    {
        return Arr::get($this->parsers, $this->getType($el), null);
    }

    public function parse(SimpleXMLIterator $element): array
    {
        $parser = $this->getParser($element);
        $parsedData = [];

        if ($parser instanceof ItemParser) {
            $parsedData = $parser->parse($element);
            if($element->properties && $parsedData[0] instanceof Item) {
                $parsedData[0]->fill(['objectProperties' => $this->objectProperties($element)]);
            }
        }

        return $parsedData;
    }
    private function objectProperties(SimpleXMLIterator $element): array
    {
        $props = [[]];
        foreach ($element->properties->property as $property) {
            $props[] = $this->getObjectProperty($property);
        }

        return array_merge(...$props);
    }
    private function getObjectProperty(SimpleXMLIterator $el): array
    {
        $properties = Arr::get((array)$el->attributes(), '@attributes', []);
        if (!isset($properties['name'])) {
            return[];
        }

        return [$properties['name'] => $properties['value'] ?? ''];
    }
}
