<?php

declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use SimpleXMLIterator;

class TableParser extends LinkParser implements ItemParser, TypeParser
{
    public const OBJECT_TYPE = 'x-im/table';

    public function parse(SimpleXMLIterator $element): array
    {
        $item = new Item($element);
        $data = $item->getFirstChild('data');

        $item->fill([
            'caption' => $this->asXml('caption', $data),
            'head' => $this->asXml('thead', $data),
            'body' => $this->asXml('tbody', $data),
            'foot' => $this->asXml('tfood', $data),
        ]);

        return [$item];
    }
    public function asXml(string $key, $data): string
    {
        $element = null;
        if ($data instanceof Item) {
            $element = $data->getFirstChild($key);
        }

        return $element instanceof Item ? $element->getContentAsXml() : '';
    }

    public function typeToParse(): string
    {
        return static::OBJECT_TYPE;
    }
}
