<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Prophecy\Argument;
use SimpleXMLIterator;
use PhpSpec\ObjectBehavior;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\NewsML\Parsers\ImageLinkParser;


/**
 * @todo Everything
 */
class ImageLinkParserSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(ImageLinkParser::class);
    }
    public function it_should_parse_an_element()
    {
        $xml = '
            <link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae">
                <data>
                    <width>300</width>
                    <height>169</height>
                </data>
                <data>
                    <width>3000</width>
                    <height>169</height>
                    <imageInstructions/>
                    <text/>
                    <alttext/>
                </data>
            </link>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveType();
        $this->parse($el)
            ->shouldHaveArrayKey([
                "rel" => "self",
                "type" => "x-im/image",
                "uri" => "im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg",
                "uuid" => "d83dad15-9476-54c1-a57d-495757664dae",
                "width" => "300",
                "height" => "169",
            ]);
    }
    public function it_should_have_default_values()
    {
        $xml = '
            <link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae"> 
            </link>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayKey([
                "rel" => "self",
                "type" => "x-im/image",
                "uri" => "im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg",
                "uuid" => "d83dad15-9476-54c1-a57d-495757664dae",
                "width" => "0",
                "height" => "0",
            ]);
    }
    public function getMatchers(): array
    {
        return [
            'haveArrayKey' => function ($subject, $values) {
                return $subject[0]->getProperties() === $values;
            },
            'haveType' => function ($subject) {
                return $subject[0] instanceof ImageItem;
            },
        ];
    }
}
