<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Prophecy\Argument;
use PhpSpec\ObjectBehavior;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Parsers\PdfParser;
use Infomaker\Everyware\NewsML\Parsers\ImageParser;
use Infomaker\Everyware\NewsML\Parsers\TableParser;
use Infomaker\Everyware\NewsML\Parsers\ObjectParser;
use Infomaker\Everyware\NewsML\Parsers\ReviewParser;
use Infomaker\Everyware\NewsML\Parsers\TeaserParser;
use Infomaker\Everyware\NewsML\Parsers\ElementParser;
use Infomaker\Everyware\NewsML\Parsers\YoutubeParser;
use Infomaker\Everyware\NewsML\Parsers\IframelyParser;
use Infomaker\Everyware\NewsML\Parsers\HtmlEmbedParser;
use Infomaker\Everyware\NewsML\Parsers\LinkObjectParser;
use Infomaker\Everyware\NewsML\Parsers\ContentPartParser;
use Infomaker\Everyware\NewsML\Parsers\SocialEmbedParser;
use Infomaker\Everyware\NewsML\Parsers\ImageGalleryParser;
use Infomaker\Everyware\NewsML\Parsers\Metadata\PersonParser;
use Infomaker\Everyware\NewsML\Parsers\Metadata\PolygonParser;
use Infomaker\Everyware\NewsML\Parsers\Metadata\ContactInfoParser;

class ObjectParserSpec extends ObjectBehavior
{
    function let()
    {
        $this->beConstructedWith([
            SocialEmbedParser::OBJECT_TYPE => new SocialEmbedParser(),
            TeaserParser::OBJECT_TYPE => new TeaserParser(),
            ImageParser::OBJECT_TYPE => new ImageParser(),
            ImageGalleryParser::OBJECT_TYPE => new ImageGalleryParser(),
            TableParser::OBJECT_TYPE => new TableParser(),
            ContentPartParser::OBJECT_TYPE => new ContentPartParser(new ElementParser()),
            ReviewParser::OBJECT_TYPE => new ReviewParser(new ElementParser()),
            LinkObjectParser::OBJECT_TYPE => new LinkObjectParser(),
            HtmlEmbedParser::OBJECT_TYPE => new HtmlEmbedParser(),
            PdfParser::OBJECT_TYPE => new PdfParser(),
            YoutubeParser::OBJECT_TYPE => new YoutubeParser(),
            ContactInfoParser::OBJECT_TYPE => new ContactInfoParser(),
            PersonParser::OBJECT_TYPE => new PersonParser(),
            PolygonParser::OBJECT_TYPE => new PolygonParser(),
            IframelyParser::OBJECT_TYPE => new IframelyParser,
        ]);
    }
    function it_is_initializable()
    {
        $this->shouldHaveType(ObjectParser::class);
    }
    public function it_should_parse_an_element()
    {
        $xml = '
            <object type="x-im/link" id="contentrelations-c0580e5c1a9fac463b6989b2bc48fb10" title="" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                <links>
                    <link rel="self" type="x-im/article" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                    </link>
                </links>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayIdAndType('54db7504-cb51-4a1a-8c62-137e465e65ba');
    }
    public function it_should_return_an_empty_array_if_parsers_are_none_existing()
    {

        $this->beConstructedWith(['x-im/link' => 'nada']);
        $xml = '
            <object type="x-im/link" id="contentrelations-c0580e5c1a9fac463b6989b2bc48fb10" title="" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                <links>
                    <link rel="self" type="x-im/article" uuid="54db7504-cb51-4a1a-8c62-137e465e65ba">
                    </link>
                </links>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldReturn([]);
    }
    public function it_should_have_object_properties()
    {
        $xml = '
            <object id="table-d23e16c0ff12dfb278e90de4999c5ae5" type="x-im/table">
                <links>
                    <link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae">
                        <data>
                            <width>300</width>
                            <height>169</height>
                            <imageInstructions/>
                            <text/>
                            <alttext/>
                        </data>
                    </link>
                </links>
                <properties>
                    <property name="alignment" value="left"/>
                    <property name="effect" value="yes"/>
                </properties>
            </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayStuff([
                "id" => "table-d23e16c0ff12dfb278e90de4999c5ae5",
                "type" => "x-im/table",
                "caption" => "",
                "head" => "",
                "body" => "",
                "foot" => "",
                "objectProperties" => [
                    "alignment" => "left",
                    "effect" => "yes",
                ]
          ]);
    }
    public function it_should_not_have_objectproperties_if_object_is_empty()
    {
        $xml = '
            <object>
                <properties>
                    <property name="alignment" value="left"/>
                    <property name="effect" value="yes"/>
                </properties>
            </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldEqual([]);
    }
    public function it_should_not_set_object_property_if_name_is_missing()
    {
        $xml = '
            <object type="x-im/youtube">
                <properties>
                    <property value="left"/>
                    <property name="effect" value="yes"/>
                </properties>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
        ->shouldHaveArrayStuff([
            "type" => "x-im/youtube",
            "uri" => "",
            "thumb" => [],
            "start" => '0',
            "objectProperties" => [
                "effect" => "yes",
            ]
      ]);
    }
    public function it_should_not_set_object_property_even_if_value_is_missing()
    {
        $xml = '
            <object type="x-im/youtube">
                <properties>
                    <property name="alignment" value=""/>
                    <property name="effect" value="yes"/>
                </properties>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
        ->shouldHaveArrayStuff([
            "type" => "x-im/youtube",
            "uri" => "",
            "thumb" => [],
            "start" => '0',
            "objectProperties" => [
                "alignment" => "",
                "effect" => "yes",
            ]
      ]);
    }
    public function getMatchers(): array
    {
        return [
            'haveArrayIdAndType' => function ($subject, string $id) {
                $properties = $subject[0]->getItemAttributes();

                return ($properties['uuid'] === $id) && ($subject[0] instanceof Item);
            },
            'haveArrayStuff' => function ($subject, array $values) {
                $properties = $subject[0]->getItemAttributes();

                return $properties === $values;
            },
        ];
    }
}
