<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\TableParser;
use Infomaker\Everyware\NewsML\Item;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class TableParserSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(TableParser::class);
    }
    public function it_should_parse_an_element_with_data()
    {
        $xml = '
        <object type="x-im/table">
            <data>
                <meta/>
                <caption><strong>Str</strong></caption>
                <tbody>
                    <tr>
                        <td>Row-1</td>
                        <td>Cell-2</td>
                    </tr>
                    <tr>
                        <td>
                            Row-2
                        </td>
                        <td>
                            <strong>strong</strong>
                        </td>
                    </tr>
                </tbody>
            </data>
        </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveTableStructure([
                'type' => 'x-im/table',
                'caption' => '<caption><strong>Str</strong></caption>',
                'head' => '',
                'body' => '
                <tbody>
                    <tr>
                        <td>Row-1</td>
                        <td>Cell-2</td>
                    </tr>
                    <tr>
                        <td>
                            Row-2
                        </td>
                        <td>
                            <strong>strong</strong>
                        </td>
                    </tr>
                </tbody>',
                'foot' => '',
            ]);
    }
    public function it_should_parse_a_caption_without_children()
    {
        $xml = '
        <object type="x-im/table">
            <data>
                <caption>Str</caption>
            </data>
        </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveTableStructure([
                'type' => 'x-im/table',
                'caption' => '<caption>Str</caption>',
                'head' => '',
                'body' => '',
                'foot' => '',
            ]);
    }
    public function it_should_have_default_values_when_data_is_empty()
    {
        $xml = '
        <object type="x-im/table">
            <data>
            </data>
        </object>
            ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveTableStructure([
                'type' => 'x-im/table',
                'caption' => '',
                'head' => '',
                'body' => '',
                'foot' => '',
            ]);
    }
    public function it_should_have_default_values_when_data_is_missing()
    {
        $xml = '
        <object type="x-im/table">
        </object>
            ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveTableStructure([
                'type' => 'x-im/table',
                'caption' => '',
                'head' => '',
                'body' => '',
                'foot' => '',
            ]);
    }
    public function it_should_return_its_type()
    {
        $this->typeToParse()->shouldReturn('x-im/table');
    }
    public function getMatchers(): array
    {
        return [
            'haveTableStructure' => function ($subject, $value) {
                $dd = $subject[0]->toArray();

                foreach ($dd as $key => $d) {
                    $m[$key] = str_replace(' ', '', $d);
                }
                foreach ($value as $key => $d) {
                    $s[$key] = str_replace(' ', '', $d);
                }
                return (str_replace(PHP_EOL, '', $m) === str_replace(PHP_EOL, '', $s)) && ($subject[0] instanceof Item);
            },
        ];
    }
}
